import logging
import sys

def init_logging():
    # Configure logging
    logging.getLogger().setLevel(logging.INFO)
    handler = logging.StreamHandler(sys.stderr)
    handler.setLevel(logging.DEBUG)
    formatter = logging.Formatter('%(asctime)s - %(name)s:%(lineno)d - %(levelname)s - %(message)s')
    handler.setFormatter(formatter)
    logging.getLogger().addHandler(handler)



import aiohttp
from typing import Any, Dict, Optional
from lotgi.models import rest


class RestClient(aiohttp.ClientSession):
    def __init__(self, base_url : Optional[str] = None, **kwargs : Dict[str, Any]):
        if base_url is None:
            base_url = os.environ.get("LOTGI_ENDPOINT", "https://lotgi.ai/api")

        token = os.environ["LOTGI_TOKEN"]
        kwargs.setdefault("headers", {}).update({
            "lotgi-token": token,
            "Content-type": "application/json",
        })
        super(base_url, **kwargs)

    async def submit(self, *
                   model : str,
                   input_file : str,
                   field : str,
                   target_cost : float,
                   target_deadline : int,
                   execution_tradeoff : str,
                   max_output_tokens : Optional[int],
                   regex : Optional[str],
                     _autodetect_cloud_credentials : bool = False,
                    ) -> rest.JobStatus:
        pass



from transformers import AutoTokenizer
from tqdm import tqdm
import pandas as pd
from dataclasses import dataclass

from lotgi.models.rest import JobExecutionOptionsInput

tqdm.pandas()

TEMPLATE = (
    "We have provided context information below. \n"
    "---------------------\n"
    "{context_str}"
    "\n---------------------\n"
    "Given this information, please answer the question: {query_str}\n"
)

def estimate_token_usage(input_url : str, prompt : str, model : str, field : str) -> JobExecutionOptionsInput:
    tokenizer = AutoTokenizer.from_pretrained(model)

    inputs = pd.read_json(input_url, lines=True)

    print("Inserting prompt into dataset...")
    inputs[field] = inputs[field].progress_apply(lambda text: TEMPLATE.format(context_str=text, query_str=prompt))

    num_prompt_tokens = len(tokenizer.encode(prompt))
    num_completions = len(inputs)

    def token_count(row) -> int:
        return len(tokenizer.encode(row))

    print("Calculating number of tokens...")
    num_dataset_tokens = inputs[field].progress_map(token_count).sum()

    return JobExecutionOptionsInput(
        model=model,
        input_token_count=(num_prompt_tokens * num_completions) + num_dataset_tokens,
        # TODO: This is an incredibly not-robust estimate.
        num_requests=num_completions,
    )



"""
There's some annyoing clobbering of the real openai package if we name this file `openai.py`
"""
import openai
from typing import overload

class _Completions:

    def __init__(self, completions):
        self.completions = completions

    def create(self, model, prompt, **kwargs):
        print("Intercepted this specific case of completion with a list of strings")
        return self.completions.create(model=model, prompt=prompt, **kwargs)

    def __getattr__(self, attr):
        return getattr(self.completions, attr)


class OpenAI(openai.OpenAI):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.completions = _Completions(self.completions)

